import colorama
from colorama import Fore, Style
colorama.init()


def color_rep(ss):
    ss = ss.strip('\n').split('\n')
    for i, line in enumerate(ss):
        if ':->' in line:
            line = line.split(':->')
            line = Fore.LIGHTMAGENTA_EX + line[0] + Style.RESET_ALL + ':->' + Fore.YELLOW + line[1] + Style.RESET_ALL
        if '|' in line:
            line = line.split('|')
            line = line[0] + Style.RESET_ALL + '|' + Fore.GREEN + line[1] + Style.RESET_ALL
        ss[i] = line
    return '\n'.join(ss)


def base_menu():
    print(color_rep("""Basic Tools help:
    qs -u  <url>              :-> open url using default browser
    qs -a  <app> [file..]     :-> open app or open file by app(for Mac OS X)
    qs -f  <file...>          :-> open file by default app
    qs -cal exp               :-> calculate exp
    qs -time                  :-> view current time"""))


def system_menu():
    print(color_rep("""System Tools help:
    qs -top                   :-> cpu and memory monitor
    qs -mktar <path...>       :-> create gzipped archive for path
    qs -untar <path>          :-> extract path.tar.*
    qs -mkzip <path...>       :-> make a zip for path
    qs -unzip <path>          :-> unzip path.zip"""))


def net_menu():
    print(color_rep("""Net Tools help:
    qs -http [ip] [-bind url] :-> start a multithread ftp server
    qs -netinfo [<domains>..] :-> get url's info which in clipboard or params 
    qs -dl [urls]             :-> download file from url(in clipboard)
    qs -upload                :-> upload your pypi library
    qs -upgrade               :-> update qs"""))


def api_menu():
    print(color_rep("""API Tools help:
    qs -trans [content]       :-> translate the content(in clipboard)
    qs -rmbg <img>            :-> remove image background
    qs -smms <img/*.md>       :-> upload img or img in markdown to sm.ms
    qs -upimg  -help          :-> upload img or img in markdown to platform
    qs -alioss -help          :-> get aliyun oss api help menu
    qs -txcos  -help          :-> get tencent cos api help menu
    qs -qiniu  -help          :-> get qiniu oss api help menu
    qs -weather [address]     :-> check weather (of address)
    qs -LG <image>            :-> make image larger(with AI)
    qs -nlp [words]           :-> Text(or in clipboard) error correction
    qs -sea <method> [msg]    :-> Get Or Post msg by Seafile.
    qs -pasteme <method> [*]  :-> get with key, [password] or post clipboard content"""))


def image_menu():
    print(color_rep("""Image Tools help:
    qs -stbg pic to [from]    :-> color replace for picture
    qs -v2gif path [sz] [fps] :-> generate gif from video
    qs -v2mp4 <video>         :-> format video to mp4
    qs -rmaudio <video>       :-> remove audio in video (return mp4 only)"""))


menu_table = {
    '-basic': base_menu,
    '-system': system_menu,
    '-net': net_menu,
    '-api': api_menu,
    '-image': image_menu
}


basic_funcs = {
    'self': 'basic',
    '-u': 'u',
    '-a': 'open_app',
    '-f': 'open_file',
    '-i': 'init',
    '-time': 'cur_time',
    '-cal': 'calculate'
}

system_funcs = {
    'self': 'system',
    '-top': 'top',
    '-mktar': 'mktar',
    '-untar': 'untar',
    '-mkzip': 'mkzip',
    '-unzip': 'unzip',
}

net_funcs = {
    'self': 'nettools',
    '-http': 'http',
    '-dl': 'download',
    '-upgrade': 'upgrade',
    '-upload': 'upload_pypi',
    '-netinfo': 'netinfo'
}

api_funcs = {
    'self': 'api',
    '-trans': 'translate',
    '-rmbg': 'remove_bg',
    '-smms': 'smms',
    '-upimg': 'up_img',
    '-alioss': 'ali_oss',
    '-qiniu': 'qiniu',
    '-txcos': 'txcos',
    '-LG': 'largeImage',
    '-weather': 'weather',
    '-nlp': 'AipNLP',
    '-sea': 'Seafile_Communicate',
    '-pasteme': 'Pasteme'
}

image_funcs = {
    'self': 'imagedeal',
    '-stbg': 'set_img_background',
    '-v2gif': 'v2gif',
    '-rmaudio': 'rmaudio',
    '-v2mp4': 'v2mp4'
}



import sys
import os


def set_img_background():
    from QuickStart_Rhy.ImageTools.ColorTools import transport_back, get_color_from_str

    try:
        img = sys.argv[2]
        if img == '-help':
            raise IndexError
        if not os.path.exists(img):
            exit('[ERROR] No Such File: %s' % img)
        to = sys.argv[3]
        try:
            frm = sys.argv[4]
        except IndexError:
            frm = '0,0,0,0'
    except IndexError:
        print('Usage: qs -stbg picture to_color [from_color: default transparency]')
        exit(0)
    else:
        to = get_color_from_str(to)
        frm = get_color_from_str(frm)
        img = transport_back(img, to, frm)
        iname = sys.argv[2].split('.')
        iname = iname[0] + '_stbg.' + ''.join(iname[1:])
        img.save(iname)


def v2gif():
    from QuickStart_Rhy.ImageTools.VideoTools import video_2_gif

    try:
        video = sys.argv[2]
        sz, fps = None, None
        if len(sys.argv) > 3:
            for i in sys.argv[3:]:
                try:
                    if ',' in i:
                        sz = tuple([int(j) for j in i.split(',')])
                    else:
                        fps = int(i)
                except:
                    raise IndexError
        sz = tuple([int(i) for i in sys.argv[3].split(',')]) if len(sys.argv) > 3 and ',' in sys.argv[3] else None
    except IndexError:
        print('Usage: qs -v2gif *.mp4 width,height fps')
        exit(0)
    else:
        video_2_gif(video, sz, fps) if sz and fps else video_2_gif(video, sz) \
            if sz else video_2_gif(video, fps) if fps else video_2_gif(video)


def rmaudio():
    from QuickStart_Rhy.ImageTools.VideoTools import rm_audio
    try:
        video = sys.argv[2]
    except IndexError:
        exit('Usage: qs -rmaudio *.mp4')
    else:
        rm_audio(video)


def v2mp4():
    from QuickStart_Rhy.ImageTools.VideoTools import tomp4
    try:
        video = sys.argv[2]
    except IndexError:
        exit('Usage: qs -v2mp4 <video>')
    else:
        tomp4(video)



from colorama import Back, Style
import math


class RollBar:
    def __init__(self, values, max_=100, height=10):
        self._title = ''
        self.max_ = max_
        self.width = len(values)
        self.height = height
        self.canvas = [[] for ignore in range(height)]
        for val in values:
            cur = math.ceil(val / max_ * height)
            for _i in range(cur):
                self.canvas[_i].append(Back.GREEN + ' ' + Style.RESET_ALL)
            for _i in range(cur, height):
                self.canvas[_i].append(Style.RESET_ALL + ' ')

    def __str__(self):
        if self.title:
            title_idx = (self.width - len(self._title)) // 2 + 1
            title_idx = max(title_idx, 0)
            res = ' ' * title_idx + self._title + '\n'
        else:
            res = ''
        return res+'100%%%s' % ('_' * (self.width+1)) + '\n    │' + '\n    │'.join(
            [''.join(i) for i in self.canvas[::-1]]) + '\n  0%╂' + '────┴' * math.ceil(self.width // 5)

    def show(self):
        print(self)

    def add(self, val):
        for _i in self.canvas:
            _i.pop(0)
        cur = math.ceil(val / self.max_ * self.height)
        for _i in range(cur):
            self.canvas[_i].append(Back.GREEN + ' ' + Style.RESET_ALL)
        for _i in range(cur, self.height):
            self.canvas[_i].append(Style.RESET_ALL + ' ')

    def title(self, _title: str):
        self._title = _title

    def set(self, vals):
        for i in self.canvas:
            i.clear()
        for val in vals:
            cur = math.ceil(val / self.max_ * self.height)
            for _i in range(cur):
                self.canvas[_i].append(Back.GREEN + ' ' + Style.RESET_ALL)
            for _i in range(cur, self.height):
                self.canvas[_i].append(Style.RESET_ALL + ' ')



import json
from QuickStart_Rhy.API import pre_check
from tencentcloud.common import credential
from tencentcloud.common.profile.client_profile import ClientProfile
from tencentcloud.common.profile.http_profile import HttpProfile
from tencentcloud.tmt.v20180321 import tmt_client, models


class Translate:
    def __init__(self, scid=pre_check('txyun_scid'), sckey=pre_check('txyun_sckey'), region=pre_check('txyun_df_region')):
        cred = credential.Credential(scid, sckey)
        http_profile = HttpProfile()
        http_profile.endpoint = "tmt.tencentcloudapi.com"
        clientProfile = ClientProfile()
        clientProfile.httpProfile = http_profile
        self.client = tmt_client.TmtClient(cred, region, clientProfile)

    def langdetect(self, text):
        req = models.LanguageDetectRequest()
        req.from_json_string(json.dumps({
            "Text": text[:len(text) // 2],
            "ProjectId": 0
        }))
        return json.loads(self.client.LanguageDetect(req).to_json_string())['Lang']

    def translate(self, text: str):
        req = models.TextTranslateRequest()
        req.from_json_string(json.dumps({
            "SourceText": text,
            "Source": self.langdetect(text),
            "Target": "zh",
            "ProjectId": 0
        }))
        return json.loads(self.client.TextTranslate(req).to_json_string())['TargetText']



from PIL import Image


def transport_back(src: str, to_color, from_color=(0, 0, 0, 0)):
    src = Image.open(src)
    src = src.convert('RGBA')
    L, H = src.size
    color_0 = from_color
    transparency_flag = False if color_0[-1] else True
    for h_indx in range(H):
        for l_indx in range(L):
            dot = (l_indx, h_indx)
            color_1 = src.getpixel(dot)
            if transparency_flag and not color_1[-1]:
                src.putpixel(dot, to_color)
            elif color_0[:-1] == color_1[:-1]:
                src.putpixel(dot, to_color)
    return src


def get_color_from_str(str_color: str):
    if ',' in str_color:
        str_color = [int(i) for i in str_color.split(',')]
        if len(str_color) == 3:
            str_color.append(255)
        str_color = tuple(str_color)
    elif len(str_color) == 6:
        str_color = (int(str_color[:2], 16), int(str_color[2:4], 16), int(str_color[4:], 16), 255)
    else:
        exit('ERROR COLOR!')
    return str_color



from QuickStart_Rhy import dir_char


def top():
    import time
    import math
    import psutil
    import colorama
    from prettytable import PrettyTable
    from colorama import Style, ansi, Cursor
    from QuickStart_Rhy import cur_time
    from QuickStart_Rhy.TuiTools import Bar
    from QuickStart_Rhy.NetTools.normal_dl import size_format

    def deal():
        print(ansi.clear_screen() + Cursor.POS(0, 0) + Style.RESET_ALL, end='')
        exit(0)

    colorama.init()
    _kernal = psutil.cpu_count()
    _total_mem = psutil.virtual_memory().total
    _cpu_dt = [0] * 40
    _mem_dt = [0] * 40
    _cpu_chart = Bar.RollBar(_cpu_dt, height=10)
    _mem_chart = Bar.RollBar(_mem_dt, height=10)
    charts = [_cpu_chart, _mem_chart]
    window = PrettyTable()
    window.add_row(charts)
    print(ansi.clear_screen())
    try:
        while True:
            _cpu_cur = sum(psutil.cpu_percent(percpu=True)) / _kernal
            _mem_cur = psutil.virtual_memory().used
            _cpu_chart.add(math.ceil(_cpu_cur))
            _mem_chart.add(math.ceil(_mem_cur / _total_mem * 100))
            window.field_names = ['CPU: %.2f%%' % _cpu_cur, 'MEM: %s' % size_format(_mem_cur)]
            print((ansi.clear_screen() if dir_char == '\\' else '') + Cursor.POS(0, 0))
            print(' ' * 39, end='')
            cur_time()
            cur_img = str(window).split('\n')
            for i in cur_img:
                print(' ' * 4, end='')
                print(i)
            time.sleep(1)
    except:
        deal()


def mktar():
    import os
    from QuickStart_Rhy.SystemTools.Compress import get_tar_name, Tar
    tar_name, ls = get_tar_name()
    tar = Tar(tar_name+'.tar.gz', 'write')

    def dfs(cur_p):
        if os.path.isfile(cur_p):
            tar.add_file(cur_p)
            return
        file_ls = os.listdir(cur_p)
        flag = cur_p.endswith(dir_char)
        for fp in file_ls:
            fp = cur_p + ('' if flag else dir_char) + fp
            dfs(fp)
    for i in ls:
        dfs(i)
    tar.save()


def untar():
    import os
    import sys

    file_names = sys.argv[2:]
    if not file_names:
        exit("No enough parameters")
    from QuickStart_Rhy.SystemTools.Compress import Tar
    from QuickStart_Rhy.NetTools.normal_dl import core_num
    from QuickStart_Rhy.ThreadTools import ThreadPoolExecutor, wait

    pool = ThreadPoolExecutor(max_workers=max(core_num // 2, 4))
    job_q = []

    def run(path):
        if os.path.exists(path):
            cur_tar = Tar(path)
            cur_tar.extract()
            del cur_tar
        else:
            print("No such file or dictionary:%s" % path)

    for file_name in file_names:
        job_q.append(pool.submit(run, file_name))
    wait(job_q)


def mkzip():
    import os
    from QuickStart_Rhy.SystemTools.Compress import get_tar_name, Zip
    zip_name, ls = get_tar_name()
    z = Zip(zip_name+'.zip', mode='write')

    def dfs(cur_p):
        if os.path.isfile(cur_p):
            z.add_file(cur_p)
            return
        file_ls = os.listdir(cur_p)
        flag = cur_p.endswith(dir_char)
        for fp in file_ls:
            fp = cur_p + ('' if flag else dir_char) + fp
            dfs(fp)
    for i in ls:
        dfs(i)
    z.save()


def unzip():
    import os
    import sys

    file_names = sys.argv[2:]
    if not file_names:
        exit("No enough parameters")
    from QuickStart_Rhy.SystemTools.Compress import Zip
    from QuickStart_Rhy.NetTools.normal_dl import core_num
    from QuickStart_Rhy.ThreadTools import ThreadPoolExecutor, wait

    pool = ThreadPoolExecutor(max_workers=max(core_num // 2, 4))
    job_q = []

    def run(path):
        if os.path.exists(path):
            try:
                z = Zip(path)
                z.extract()
            except:
                print('[ERROR] %s extract failed!' % path)
        else:
            print("No such file or dictionary:%s" % path)

    for file_name in file_names:
        job_q.append(pool.submit(run, file_name))
    wait(job_q)



import diagram
import io


class Line:
    def __init__(self, values, height, width):
        self.values = values
        self.width = height
        self.height = width

    def __str__(self):
        opt = diagram.DOption()
        opt.size = diagram.Point([self.height, self.width])
        opt.mode = 'g'
        stream = io.BytesIO()
        gram = diagram.DGWrapper(
            data=[self.values, range(len(self.values))],
            dg_option=opt,
            ostream=stream
        )
        gram.show()
        return str(stream.getvalue(), encoding="utf-8")

    def push(self, x):
        self.values.append(x)

    def pop(self):
        self.values.pop(0)



import sys
from QuickStart_Rhy.func_list import *


def qs_help(rep=''):
    if not rep or rep not in menu_table:
        print('help:')
        print(color_rep(
            """
    qs -basic   :-> basic   tools help | -u, -a, -f, -cal, -time
    qs -system  :-> system  tools help | -top, -[mk/un][zip/tar]
    qs -net     :-> network tools help | -ftp, -dl, -up[load/grade]
    qs -api     :-> api     tools help | -trans, -smms, -rmbg...
    qs -image   :-> image   tools help | -stbg, -v2gif, -v2mp4...
"""))
        print('Docs:\n  -China: ' + Fore.CYAN + 'https://rhythmlian.cn/2020/02/14/QuickStart-Rhy/' + Style.RESET_ALL)
        print('  -Other: ' + Fore.CYAN + 'https://rhythmicc.github.io/2020/02/14/QuickStart-Rhy/' + Style.RESET_ALL)
    else:
        menu_table[rep]()


cmd_config = {}
for i in basic_funcs:
    if i.startswith('-'):
        cmd_config[i] = basic_funcs
for i in api_funcs:
    if i.startswith('-'):
        cmd_config[i] = api_funcs
for i in net_funcs:
    if i.startswith('-'):
        cmd_config[i] = net_funcs
for i in image_funcs:
    if i.startswith('-'):
        cmd_config[i] = image_funcs
for i in system_funcs:
    if i.startswith('-'):
        cmd_config[i] = system_funcs


def main():
    if len(sys.argv) >= 2:
        func_name = sys.argv[1]
        if func_name not in cmd_config:
            qs_help(func_name)
        else:
            func_table = cmd_config[func_name]
            file_name = func_table['self']
            func_name = func_table[func_name]
            if file_name == 'basic':
                exec('from QuickStart_Rhy import %s' % func_name)
            else:
                exec('from QuickStart_Rhy.%s import %s' % (file_name, func_name))
            eval('%s()' % func_name)
    else:
        qs_help()


if __name__ == '__main__':
    main()



import os
import sys


def get_console_width():
    if os.name == 'nt':
        STD_OUTPUT_HANDLE = -11
        from ctypes import windll, Structure, byref
        from ctypes.wintypes import SHORT, WORD, DWORD
        console_handle = windll.kernel32.GetStdHandle(STD_OUTPUT_HANDLE)

        # CONSOLE_SCREEN_BUFFER_INFO Structure
        class COORD(Structure):
            _fields_ = [("X", SHORT), ("Y", SHORT)]

        class SMALL_RECT(Structure):
            _fields_ = [("Left", SHORT), ("Top", SHORT),
                        ("Right", SHORT), ("Bottom", SHORT)]

        class CONSOLE_SCREEN_BUFFER_INFO(Structure):
            _fields_ = [("dwSize", COORD),
                        ("dwCursorPosition", COORD),
                        ("wAttributes", WORD),
                        ("srWindow", SMALL_RECT),
                        ("dwMaximumWindowSize", DWORD)]

        sbi = CONSOLE_SCREEN_BUFFER_INFO()
        ret = windll.kernel32.GetConsoleScreenBufferInfo(console_handle, byref(sbi))
        if ret == 0:
            return 0
        return sbi.srWindow.Right + 1

    elif os.name == 'posix':
        from fcntl import ioctl
        from termios import TIOCGWINSZ
        from array import array

        winsize = array("H", [0] * 4)
        try:
            ioctl(sys.stdout.fileno(), TIOCGWINSZ, winsize)
        except IOError:
            pass
        return (winsize[1], winsize[0])[0]

    return 80



import io
import os
import sys
import tarfile
import zipfile
from QuickStart_Rhy import dir_char


def get_tar_name():
    file_names = sys.argv[2:]
    if not file_names:
        exit('No enough parameters')
    if len(file_names) > 1:
        tar_name = 'pigeonhole'
    else:
        ls = file_names[0].split(dir_char)
        while not ls[-1]:
            ls.pop()
        tar_name = ls[-1].split('.')[0]
    ls = []
    for file_name in file_names:
        if os.path.exists(file_name):
            ls.append(file_name)
        else:
            print("No such file or dictionary:%s" % file_name)
    return tar_name, ls


class Tar:
    def __init__(self, path, mode='read'):
        if mode == 'read':
            if os.path.exists(path):
                self.src = tarfile.open(path, 'r')
                self.mode = True
            else:
                raise FileNotFoundError
        elif mode == 'write':
            self.src = tarfile.open(path, 'x:gz')
            self.mode = False

    def add_file(self, path):
        if self.mode:
            raise io.UnsupportedOperation
        if os.path.exists(path):
            self.src.add(path)
        else:
            raise FileNotFoundError

    def extract(self):
        if self.mode:
            self.src.extractall()
        else:
            raise io.UnsupportedOperation

    def save(self):
        self.src.close()


class Zip:
    def __init__(self, path, mode='read'):
        if mode == 'read':
            if os.path.exists(path):
                self.src = zipfile.ZipFile(path, 'r')
                self.mode = True
            else:
                raise FileNotFoundError
        elif mode == 'write':
            if os.path.exists(path):
                raise io.UnsupportedOperation('File exists!')
            self.src = zipfile.ZipFile(path, 'w')
            self.mode = False

    def add_file(self, path):
        if self.mode:
            raise io.UnsupportedOperation('not writeable')
        if os.path.exists(path):
            self.src.write(path)
        else:
            raise FileNotFoundError

    def extract(self):
        if self.mode:
            self.src.extractall()
        else:
            raise io.UnsupportedOperation('not readable')

    def save(self):
        self.src.close()



def set_timeout(num):
    def wrapper(func):
        def handle(signum, frame):
            raise RuntimeError

        def run(*args, **kwargs):
            import signal
            try:
                signal.signal(signal.SIGALRM, handle)
                signal.alarm(num)
                res = func(*args, **kwargs)
                signal.alarm(0)
                return res
            except RuntimeError:
                return False
        return run
    return wrapper



from QuickStart_Rhy.API import *
import qiniu


class Qiniu_oss_api:
    def __init__(self):
        self.ac_key = pre_check('qiniu_ac_key')
        self.sc_key = pre_check('qiniu_sc_key')
        self.auth = qiniu.Auth(self.ac_key, self.sc_key)
        self.df_bucket = pre_check('qiniu_bk_name')

    def get_func_table(self):
        return {
            '-up': self.upload,
            '-rm': self.remove,
            '-cp': self.copy_url,
            '-dl': self.download,
            '-ls': self.list_bucket
        }

    def upload(self, filePath: str, bucket=None):
        tk = self.auth.upload_token(bucket if bucket else self.df_bucket, filePath.split(dir_char)[-1])
        qiniu.put_file(tk, filePath.split(dir_char)[-1], filePath)

    def remove(self, filePath: str, bucket=None):
        bk = qiniu.BucketManager(self.auth)
        bk.delete(bucket if bucket else self.df_bucket, filePath)

    def copy_url(self, filePath: str, bucket=None):
        bk = qiniu.BucketManager(self.auth)
        bk.fetch(filePath, bucket if bucket else self.df_bucket, filePath.split('/')[-1])

    def get_bucket_url(self, bucket=None):
        import requests
        bucket = bucket if bucket else self.df_bucket
        url = 'http://api.qiniu.com/v6/domain/list?tbl=%s' % bucket
        headers = {
            "Content-Type": "application/x-www-form-urlencoded",
            "Authorization": "QBox %s" % self.auth.token_of_request(url)
        }
        res = requests.get(url, headers=headers)
        if res.status_code == requests.codes.ok:
            return res.json()
        else:
            return False

    def list_bucket(self, bucket=None):
        from prettytable import PrettyTable
        bk = qiniu.BucketManager(self.auth)
        ret = bk.list(bucket if bucket else self.df_bucket)
        if not ret[1]:
            print("ERROR!")
            exit(0)
        root_url = 'http://' + self.get_bucket_url(bucket)[0] + '/'
        ret = ret[0]['items']
        from QuickStart_Rhy.NetTools.normal_dl import size_format
        tb = PrettyTable(['File', 'Size'])
        for i in ret:
            tb.add_row([i['key'], size_format(i['fsize'])])
        print("Bucket url:", root_url)
        print(tb)

    def download(self, filePath: str, bucket=None):
        from QuickStart_Rhy.NetTools.normal_dl import normal_dl
        bucket = bucket if bucket else self.df_bucket
        root_url = self.get_bucket_url(bucket)[0]
        if root_url:
            root_url = 'http://' + root_url + '/'
        else:
            exit('Get Bucket Url Failed!')
        dl_url = root_url + filePath
        if bucket.startswith('admin'):
            dl_url = self.auth.private_download_url(dl_url)
        normal_dl(dl_url)



from QuickStart_Rhy.API import pre_check
import pyperclip


class Seafile:
    def __init__(self, path=pre_check('seafile_communicate_path')):
        self.path = path

    def get_msg(self):
        with open(self.path, 'rb') as f:
            pyperclip.copy(f.read().decode('utf-8'))

    def post_msg(self, msg=pyperclip.paste()):
        if not msg:
            raise IndexError
        with open(self.path, 'wb') as f:
            f.write(msg.encode('utf-8'))



import sys


def remove_bg():
    try:
        path = sys.argv[2]
    except IndexError:
        exit('Usage: qs -rmbg picture')
    else:
        if path == '-help':
            print('Usage: qs -rmbg picture')
            return
        from QuickStart_Rhy.API.simple_api import rmbg
        rmbg(path)


def smms():
    try:
        path = sys.argv[2]
    except IndexError:
        exit('Usage: qs -smms [picture | *.md]')
    else:
        if path == '-help':
            print('Usage: qs -smms [picture | *.md]')
            return
        from QuickStart_Rhy.API.simple_api import smms
        smms(path)


def up_img():
    try:
        path = sys.argv[2]
    except IndexError:
        exit('Usage: qs -upimg [picture]')
    else:
        from QuickStart_Rhy.API.simple_api import upimg
        import random
        spt_type = {'Ali': '阿里云', 'Sogou': '搜狗', 'Vimcn': 'Vim-cn.com', 'Niupic': '牛图', 'Juejin': '掘进',
                    'UploadLiu': 'upload.ouliu.net', 'Catbox': 'catbox', 'NetEasy': '网易', 'Prnt': 'Prnt',
                    'Qihoo': '360奇虎', 'Souhu': '搜狐', 'Toutiao': '头条', 'Xiaomi': '小米', 'ImgTg': 'imt.tg'}
        spt_type_keys = list(spt_type.keys())
        if path == '-help':
            print('Usage: qs -upimg <picture | *.md> [platform]\n\nSupport ([platform]: description):')
            print(''.join(['%-10s: %s%s' % (spt_type_keys[i], spt_type[spt_type_keys[i]], '\t' if (i + 1) % 3 else '\n')
                           for i in range(len(spt_type_keys))]))
            print('\n[NOTE] If you do not set platform, qs will randomly choose one.')
            return
        type_map = {}
        for i in spt_type:
            type_map[i.lower()] = i
            type_map[i] = i
        upimg(path, type_map[sys.argv[3]]) if len(sys.argv) > 3 and sys.argv[3] in type_map \
            else upimg(path, random.choice(spt_type_keys)), \
            print('No such platform: %s' % sys.argv[3]) if len(sys.argv) > 3 else 1


def ali_oss():
    try:
        op = sys.argv[2]
        if op not in ['-dl', '-up', '-ls', '-rm']:
            raise IndexError
        file = sys.argv[3] if op != '-ls' else None
        try:
            bucket = sys.argv[4] if op != '-ls' else sys.argv[3]
        except IndexError:
            bucket = None
    except IndexError:
        print('qs -alioss:\n'
              '\t-up <file> [bucket]: upload file to bucket\n'
              '\t-dl <file> [bucket]: download file from bucket\n'
              '\t-rm <file> [bucket]: remove file in bucket\n'
              '\t-ls [bucket]       : get file info of bucket')
        exit(0)
    else:
        from QuickStart_Rhy.API.Aliyun_oss import Aliyun_oss_api
        ali_api = Aliyun_oss_api()
        func_table = ali_api.get_func_table()
        if not file:
            func_table[op](bucket)
        else:
            func_table[op](file, bucket)


def qiniu():
    try:
        op = sys.argv[2]
        if op not in ['-up', '-rm', '-cp', '-ls', '-dl']:
            raise IndexError
        file = sys.argv[3] if op != '-ls' else None
        try:
            bucket = sys.argv[4] if op != '-ls' else sys.argv[3]
        except IndexError:
            bucket = None
    except IndexError:
        print('qs -qiniu:\n'
              '\t-up <file> [bucket]: upload file to bucket\n'
              '\t-rm <file> [bucket]: remove file in bucket\n'
              '\t-cp <url > [bucket]: copy file from url\n'
              '\t-dl <file> [bucket]: download file from bucket\n'
              '\t-ls [bucket]       : get file info of bucket')
        exit(0)
    else:
        from QuickStart_Rhy.API.Qiniu_oss import Qiniu_oss_api
        qiniu_api = Qiniu_oss_api()
        func_table = qiniu_api.get_func_table()
        if not file:
            func_table[op](bucket)
        else:
            func_table[op](file, bucket)


def txcos():
    try:
        op = sys.argv[2]
        if op not in ['-dl', '-up', '-ls', '-rm']:
            raise IndexError
        file = sys.argv[3] if op != '-ls' else None
        try:
            bucket = sys.argv[4] if op != '-ls' else sys.argv[3]
        except IndexError:
            bucket = None
    except IndexError:
        print('qs -txcos:\n'
              '\t-up <file> [bucket]: upload file to bucket\n'
              '\t-dl <file> [bucket]: download file from bucket\n'
              '\t-rm <file> [bucket]: remove file in bucket\n'
              '\t-ls [bucket]       : get file info of bucket')
        exit(0)
    else:
        from QuickStart_Rhy.API.txcos import txcos
        tx_api = txcos()
        func_table = tx_api.get_func_table()
        if not file:
            func_table[op](bucket)
        else:
            func_table[op](file, bucket)


def translate():
    import pyperclip
    from QuickStart_Rhy.API.TencentTranslate import Translate

    content = ' '.join(sys.argv[2:])
    if not content:
        content = pyperclip.paste()
    if content:
        content.replace('\n', ' ')
        translator = Translate()
        ret = translator.translate(content)
        print(ret if ret else 'Translate Failed!')
    else:
        print("No content in your clipboard or command parameters!")


def weather():
    from QuickStart_Rhy import headers, dir_char
    from QuickStart_Rhy.ThreadTools import ThreadFunctionWrapper
    import requests

    def get_data(url):
        try:
            ct = requests.get(url, headers)
        except:
            return
        ct.encoding = 'utf-8'
        ct = ct.text.split('\n')
        if dir_char == '/':
            res = ct.copy()
        else:
            import re
            for line in range(len(ct)):
                ct[line] = re.sub('\x1b.*?m', '', ct[line])
            res = ct.copy()
        return res

    try:
        loc = sys.argv[2]
    except IndexError:
        loc = ''
    tls = [ThreadFunctionWrapper(get_data, 'https://wttr.in/' + (loc if loc else '?lang=zh')),
           ThreadFunctionWrapper(get_data, 'https://v2.wttr.in/' + loc)]
    for i in tls:
        i.start()
    for i in tls:
        i.join()
    simple = tls[0].get_res()
    table = tls[1].get_res()
    if simple:
        if not loc:
            from QuickStart_Rhy.API import pre_check
            if pre_check("txyun_scid", False) and pre_check("txyun_sckey", False):
                from QuickStart_Rhy.API.TencentTranslate import Translate
                translator = Translate()
                trans_loaction = translator.translate(simple[0].split('：')[-1])
                print('地区：' + trans_loaction if trans_loaction else simple[0].split('：')[-1])
            else:
                print('地区：' + simple[0].split('：')[-1])
        simple = simple[2:7]
        print('\n'.join(simple))
    else:
        print('Error: Get data failed.')
    if table:
        print(table[3][:-1])
        bottom_line = 7
        try:
            while '╂' not in table[bottom_line]:
                bottom_line += 1
        except IndexError:
            exit('Get Weather Data failed!')
        for i in table[7:bottom_line + 2]:
            print(i[:-1])
        print('└────────────────────────────────────────────────────────────────────────')
        print('\n'.join(table[-3 if not loc else -4:]))
    else:
        print('Error: Get detail failed.')


def ipinfo(ip: str = None):
    from QuickStart_Rhy.API.IPinfo import get_ip_info
    return get_ip_info(ip)


def largeImage():
    try:
        path = sys.argv[2]
    except IndexError:
        exit('Usage qs -LG img')
    else:
        from QuickStart_Rhy.API import AipImage
        aip_cli = AipImage.ImageDeal()
        aip_cli.largeImage(path)


def AipNLP():
    from QuickStart_Rhy.API.AipNLP import AipNLP
    import pyperclip
    ct = sys.argv[2:]
    if not ct:
        ct = [pyperclip.paste()]
    NLP = AipNLP()
    for id, line in enumerate(ct):
        ct[id] = NLP.get_res(line)
        if id == 9:
            print('...')
        elif id < 9:
            print(ct[id])
    pyperclip.copy('\n'.join(ct))


def Seafile_Communicate():
    from QuickStart_Rhy.API.Seafile import Seafile
    try:
        method = sys.argv[2]
        if method == 'get':
            Seafile().get_msg()
        elif method == 'post':
            msg = ' '.join(sys.argv[3:]) if len(sys.argv) > 3 else None
            Seafile().post_msg(msg) if msg else Seafile().post_msg()
    except IndexError:
        print("Usage:\n  1. qs -sea get\n  2. qs -sea post [msg]")
        exit(0)


def Pasteme():
    from QuickStart_Rhy.API.simple_api import pasteme
    try:
        method = sys.argv[2]
        key = sys.argv[3]
        password = sys.argv[4] if len(sys.argv) > 4 else ''
        pasteme(key, password, method)
    except IndexError:
        print("Usage:\n  1. qs -pasteme get key [password]\n  2. qs -pasteme post lang [password]")
        exit(0)



import sys


def upgrade():
    import os
    if os.system('pip3 install QuickStart-Rhy --upgrade'):
        os.system('pip install QuickStart-Rhy --upgrade')


def upload_pypi():
    import os
    from QuickStart_Rhy import remove, dir_char
    remove('dist')
    if os.system('python3 setup.py sdist'):
        os.system('python setup.py sdist')
    os.system('twine upload dist%s*' % dir_char)


def m3u8_dl(url):
    from QuickStart_Rhy.NetTools.m3u8_dl import M3U8DL
    M3U8DL(url, url.split('.')[-2].split('/')[-1]).download()


def download():
    ytb_flag = '--ytb' in sys.argv
    if ytb_flag:
        sys.argv.remove('--ytb')
    urls = sys.argv[2:]
    if not urls:
        import pyperclip
        urls = pyperclip.paste().split()
    if urls:
        if ytb_flag:
            from youtube_dl import _real_main
        from QuickStart_Rhy.NetTools.normal_dl import normal_dl
        for url in urls:
            if url.endswith('.m3u8'):
                m3u8_dl(url)
            else:
                normal_dl(url) if not ytb_flag else _real_main([url])
    else:
        print("No url found!")


def http():
    url = ''
    if len(sys.argv) > 2:
        ip, port = sys.argv[2].split(':')
        port = int(port)
        if '-bind' in sys.argv:
            try:
                url = sys.argv[sys.argv.index('-bind') + 1]
                from QuickStart_Rhy.NetTools import formatUrl
                url = formatUrl(url)
            except IndexError:
                print('Usage: qs -ftp ip:port -bind url')
                exit(0)
    else:
        from QuickStart_Rhy.NetTools import get_ip
        ip = get_ip()
        port = 8000
    if not ip:
        exit('get ip failed!')
    from QuickStart_Rhy.NetTools.server import HttpServers
    HttpServers(ip, port, url).start()


def netinfo():
    import json
    import socket
    import requests
    import pyperclip
    import prettytable
    import urllib.parse
    from QuickStart_Rhy import headers

    def print_ip_info(info_ls):
        table = prettytable.PrettyTable(['ip', '运营商', '地址', '经纬'])
        for info in info_ls:
            table.add_row([info['ip'], info['isp'], info['pos'], str(info['location'])[1:-1].replace("'", '')])
        print(table)

    urls = sys.argv[2:] if len(sys.argv) > 2 else []
    urls += pyperclip.paste().strip().split() if not urls else []
    if not urls:
        urls.append('me')
    res_ls = []
    for i in urls:
        try:
            addr = ''
            if i != 'me':
                addr = urllib.parse.urlparse(i).netloc
                addr = socket.getaddrinfo(addr if addr else i, 'http')[0][-1][0]
            res = requests.post('https://v1.alapi.cn/api/ip', data="ip=%s&format=json" % addr,
                                headers={'Content-Type': "application/x-www-form-urlencoded"} if addr else headers)
            res = json.loads(res.text)['data']
            res_ls.append(res)
        except:
            print('[ERROR] Get domain failed:', i)
            continue
    print_ip_info(res_ls)



from QuickStart_Rhy.NetTools import size_format, get_fileinfo
from concurrent.futures import ThreadPoolExecutor, wait
from QuickStart_Rhy.ThreadTools import FileWriters
from threading import Lock
from requests import get
import psutil
import signal
import queue
import time
import os

core_num = psutil.cpu_count()
maxBlockSize = int(1.5e6)
minBlockSize = int(5e5)


def GetBlockSize(sz):
    if sz >= maxBlockSize << 10:
        return maxBlockSize
    elif sz <= minBlockSize << 3:
        return minBlockSize
    else:
        return max(sz >> 9, minBlockSize)


class Downloader:
    def __init__(self, url: str, num: int):
        signal.signal(signal.SIGINT, self._kill_self)
        self.has_ctrl = False
        self.url = url
        self.num = num
        self.name = os.path.basename(url)
        self.fileLock = Lock()
        self.cur_sz = 0
        self.url, self.name, r = get_fileinfo(url)
        if not self.url:
            print('[ERROR] Connection Error!')
            exit(0)
        try:
            self.size = int(r.headers['Content-Length'])
            if self.size < 5e6:
                print('[INFO] FILE SIZE\t{}'.format(size_format(self.size)))
                self.size = -self.size
            else:
                self.fileBlock = GetBlockSize(self.size)
        except KeyError:
            self.size = -1
        if self.size > 0:
            self.pool = ThreadPoolExecutor(max_workers=self.num)
            self.futures = []
            self.job_queue = queue.Queue()
            if os.path.exists(self.name + '.qs_dl'):
                self.ctn_file = open(self.name + '.qs_dl', 'r+')
                self.ctn = [int(i) for i in self.ctn_file.read().strip().split()]
            else:
                self.ctn_file = open(self.name + '.qs_dl', 'w')
                self.ctn = []
            self.writers = FileWriters(self.name, max(2, int(core_num / 2)), "rb+" if self.ctn else "wb")
            print('[INFO] FILE SIZE\t{}'.format(size_format(self.size, align=True)))
            print('[INFO] BLOCK SIZE\t{}'.format(size_format(self.fileBlock, align=True)))

    def _kill_self(self, signum, frame):
        if self.size > 0:
            print('\n[INFO] Get Ctrl-C, exiting...')
            self.ctn_file.close()
            self.pool.shutdown()
            self.writers.wait()
            print('[INFO] Deal Done!')
        os._exit(0)

    def _dl(self, start):
        try:
            _sz = min(start + self.fileBlock, self.size)
            headers = {'Range': 'bytes={}-{}'.format(start, _sz)}
            tm = time.perf_counter()
            r = get(self.url, headers=headers, timeout=50)
            tm = time.perf_counter() - tm
            self.writers.new_job(r.content, start)
        except Exception as e:
            msg = repr(e)
            print('\n[ERROR] %s' % msg[:msg.index('(')])
            self.job_queue.put(start)
        else:
            self.fileLock.acquire()
            self.ctn.append(start)
            self.ctn_file.write('%d\n' % start)
            self.cur_sz += _sz - start
            self.fileLock.release()
            speed = size_format((self.fileBlock * self.num / tm), align=True)
            per = self.cur_sz / self.size
            print('\r[%s] %.2f%% | %s/s' % (
                    '#' * int(40 * per) + ' ' * int(40 - 40 * per),
                    per * 100, speed
                ),
                end='\n' if self.cur_sz == self.size else ''
            )

    def _single_dl(self):
        r = get(self.url, stream=True)
        Flag = self.size != -1
        self.size = -self.size
        with open(self.name, 'wb') as f:
            for chunk in r.iter_content(8192):
                f.write(chunk)
                self.cur_sz += 8192
                if Flag:
                    self.cur_sz = min(self.cur_sz, self.size)
                print('\r[PROC] %s' % size_format(self.cur_sz, align=True), end='')
            print('')
            if Flag and self.cur_sz < self.size:
                print('[ERROR] Data loss!')

    def run(self):
        if self.size > 0:
            if not self.ctn:
                with open(self.name, "wb") as fp:
                    fp.truncate(self.size)
            for i in range(0, self.size, self.fileBlock):
                if self.ctn and i in self.ctn:
                    self.cur_sz += min(i + self.fileBlock, self.size) - i
                else:
                    self.job_queue.put(i)
            retry_cnt = 0
            while not self.job_queue.empty():
                self.futures.clear()
                while not self.job_queue.empty():
                    cur = self.job_queue.get()
                    if cur not in self.ctn:
                        self.futures.append(self.pool.submit(self._dl, cur))
                wait(self.futures)
                if not self.job_queue.empty() and retry_cnt > 2:
                    print('\r[INFO] Exists File Block Lost, Retrying after 0.5 sec')
                    time.sleep(0.5)
                retry_cnt += 1
            self.writers.wait()
            self.ctn_file.close()
            os.remove(self.name + '.qs_dl')
        else:
            self._single_dl()
        print('[INFO] %s download done!' % self.name)


def normal_dl(url):
    Downloader(url, min(16, core_num * 4)).run()



import threading
from concurrent.futures import ThreadPoolExecutor, wait


class ThreadFunctionWrapper(threading.Thread):
    def __init__(self, function, *args, **kwargs):
        threading.Thread.__init__(self)
        self.function = function
        self.args = args
        self.kwargs = kwargs
        self.res = None

    def run(self):
        self.res = self.function(*self.args, **self.kwargs)

    def get_res(self):
        return self.res


class FileWriters:
    def __init__(self, filename, workers, mode):
        self.workers = workers
        self.handles = [open(filename, mode) for i in range(workers)]
        self.handles_lock = [threading.Lock() for i in range(workers)]
        self.pool = ThreadPoolExecutor(max_workers=workers)
        self.job_q = []
        self.cur_handle = 0

    def wait(self):
        wait(self.job_q)

    def _write(self, handle: int, content, index):
        fp = self.handles[handle]
        self.handles_lock[handle].acquire()
        fp.seek(index)
        fp.write(content)
        self.handles_lock[handle].release()

    def new_job(self, content, index):
        self.job_q.append(self.pool.submit(self._write, self.cur_handle, content, index))
        self.cur_handle = (self.cur_handle+1) % self.workers

    def __del__(self):
        self.wait()
        self.pool.shutdown()



import requests
from requests.exceptions import RequestException
from QuickStart_Rhy import headers


def check_one_page(url):
    try:
        response = requests.head(url, headers=headers).status_code
        return response == requests.codes.ok
    except RequestException:
        return False


def formatUrl(try_url):
    if try_url.startswith('http://') or try_url.startswith('https://'):
        return try_url
    res_url = try_url
    if not check_one_page(res_url):
        res_url = 'https://' + try_url
        if not check_one_page(res_url):
            res_url = 'http://' + try_url
    return res_url


def get_ip():
    import socket
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
        return ip
    except Exception:
        s.close()
        return socket.gethostbyname(socket.gethostname())


def size_format(sz, align=False):
    if sz >= 1e9:
        return '%.3f GB' % (sz / 1e9) if not align else '%7.3f GB' % (sz / 1e9)
    elif sz >= 1e6:
        return '%.3f MB' % (sz / 1e6) if not align else '%7.3f MB' % (sz / 1e6)
    elif sz >= 1e3:
        return '%.3f KB' % (sz / 1e3) if not align else '%7.3f KB' % (sz / 1e3)
    else:
        return '%.2f B' % sz if not align else '%7.2f B' % sz


def get_ip_info():
    import json
    res = requests.get('http://ip-api.com/json/', headers=headers)
    if res.status_code == requests.codes.ok:
        return json.loads(res.text)
    else:
        return None


def get_fileinfo(url):
    import re
    import os
    try:
        res = requests.head(url, headers=headers)
    except:
        return False, False, False
    while res.status_code == 302 or res.status_code == 301:
        url = res.headers['Location']
        res = requests.head(url, headers=headers)
    if 'Content-Disposition' in res.headers:
        try:
            filename = re.findall('filename=(.*?);', res.headers['Content-Disposition'])[0]
        except IndexError:
            from urllib.parse import urlparse
            filename = os.path.basename(urlparse(url).path.strip('/'))
    else:
        from urllib.parse import urlparse
        filename = os.path.basename(urlparse(url).path.strip('/'))
    return url, re.sub(r"^\W+|\W+$", "", filename), res



from QuickStart_Rhy.API import pre_check, os
try:
    import aip
except ImportError:
    exit('You need install: baidu-aip')
import base64


class ImageDeal:
    def __init__(self, app_id=pre_check('AipImageAPP_ID'),
                 app_key=pre_check('AipImageAPP_KEY'),
                 secret_key=pre_check('AipImageSECRET_KEY')):
        self.client = aip.AipImageProcess(app_id, app_key, secret_key)

    def largeImage(self, path):
        if not os.path.exists(path) or not os.path.isfile(path):
            print('No file named: %s' % path)
            return
        img_name = os.path.basename(path)
        img_name = img_name[:img_name.index('.')] + '_LG.' + '.'.join(img_name.split('.')[1:])
        with open(path, 'rb') as f:
            img = f.read()
        img = self.client.imageQualityEnhance(img)
        try:
            img = base64.b64decode(img['image'])
            with open(img_name, 'wb') as f:
                f.write(img)
        except :
            from pprint import pprint
            pprint(img)



import os
import sys


name = 'QuickStart_Rhy'
headers = {
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_2) AppleWebKit/604.4.7 (KHTML, like Gecko) '
                  'Version/11.0.2 Safari/604.4.7'}
system = sys.platform
if system.startswith('win'):
    dir_char = '\\'
else:
    dir_char = '/'


def deal_ctrl_c(signum, frame):
    if signum or frame or True:
        exit(0)


def remove(path):
    if os.path.exists(path):
        if os.path.isdir(path):
            import shutil
            shutil.rmtree(path)
        else:
            os.remove(path)


def cur_time():
    week = {
        'Monday': '周一',
        'Tuesday': '周二',
        'Wednesday': '周三',
        'Thursday': '周四',
        'Friday': '`周五',
        'Saturday': '周六',
        'Sunday': '周日'
    }
    import time
    tm = time.strftime('%Y年%m月%d日 %A %H:%M:%S', time.localtime(time.time())).split()
    tm[1] = week[tm[1]]
    print(' '.join(tm))


def u():
    import webbrowser as wb
    from QuickStart_Rhy.NetTools import formatUrl
    if sys.argv[2:]:
        for url in sys.argv[2:]:
            url = formatUrl(url)
            wb.open_new_tab(url)
    else:
        import pyperclip
        url = pyperclip.paste()
        wb.open_new_tab(formatUrl(url))


def open_app():
    if system == 'darwin':
        os.system('open -a ' + ' '.join(sys.argv[2:]))
    else:
        print('"-a" is only support Mac OS X')


def open_file():
    import webbrowser as wb
    if system == 'darwin':
        os.system('open "' + '" "'.join(sys.argv[2:]) + '"')
    else:
        for file in sys.argv[2:]:
            if os.path.exists(file):
                path = os.path.abspath(file)
                wb.open('file://%s' % path)


def init():
    import webbrowser as wb
    wb.open('http://login.cup.edu.cn')


def calculate():
    try:
        exp = ' '.join(sys.argv[2:])
        print('%s = %s' % (exp, eval(exp)))
    except Exception as e:
        exit('[ERROR] %s' % repr(e))



import urllib3
from concurrent.futures import ThreadPoolExecutor, wait
import requests
import os
from QuickStart_Rhy import dir_char, remove, headers
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


def merge_file(path, ts_ls, name):
    if not path.endswith(dir_char):
        path += dir_char
    with open(name + '.ts', 'wb') as f:
        for ts in ts_ls:
            with open(path + ts, 'rb') as ff:
                f.write(ff.read())


class M3U8DL:
    def __init__(self, target, name):
        self.path = ''
        self._cur = 0
        self._all = 0
        self.target = target
        self.name = name

    def _dl_one(self, job):
        if job[-1]:
            pd_url = job[0]
            c_fule_name = job[1]
            if not os.path.exists(os.path.join(self.path, c_fule_name)):
                res = requests.get(pd_url, verify=False)
                with open(os.path.join(self.path, c_fule_name), 'ab') as f:
                    f.write(res.content)
                    f.flush()
        self._cur += 1
        perc = self._cur / self._all
        leng = int(perc * 40)
        perc *= 100
        print('[%s] %.2f%%' % ('#' * leng + ' ' * (40 - leng), perc),
              end='\n' if self._cur == self._all else '\r')

    def download(self):
        target = self.target
        download_path = os.getcwd() + dir_char + self.name
        self.path = download_path
        if not os.path.exists(download_path):
            os.mkdir(download_path)
        all_content = requests.get(target, verify=False, headers=headers).text
        if "#EXTM3U" not in all_content:
            raise BaseException("非M3U8的链接")
        if "EXT-X-STREAM-INF" in all_content:
            file_line = all_content.split("\n")
            for line in file_line:
                if '.m3u8' in line:
                    target = target.rsplit("/", 1)[0] + "/" + line
                    all_content = requests.get(target, verify=False, headers=headers).text
        file_line = all_content.split("\n")
        _rt = target.rsplit("/", 1)[0] + "/"
        tmp = []
        for index, line in enumerate(file_line):
            if "#EXT-X-KEY" in line:
                tmp.append((_rt, line, 0))
            if "EXTINF" in line:
                tmp.append((_rt + file_line[index + 1], file_line[index + 1].rsplit("/", 1)[-1], 1))
        file_line = tmp[::-1]
        self._all = len(file_line)
        tmp = [jb[1] for jb in file_line]
        pool = ThreadPoolExecutor(16)
        work = [pool.submit(self._dl_one, job) for job in file_line]
        wait(work)
        print("Download completed!")
        merge_file(download_path, tmp, self.name)
        remove(download_path)



from QuickStart_Rhy import dir_char
from QuickStart_Rhy.API import pre_check
import qcloud_cos


class txcos:
    def __init__(self):
        scid = pre_check('txyun_scid')
        sckey = pre_check('txyun_sckey')
        self.region = pre_check('txyun_df_region')
        self.df_bucket = pre_check('txyun_cos_df_bucket')
        config = qcloud_cos.CosConfig(Region=self.region,
                                      SecretId=scid,
                                      SecretKey=sckey)
        self.client = qcloud_cos.CosS3Client(config)

    def get_func_table(self):
        return {
            '-up': self.upload,
            '-rm': self.remove,
            '-dl': self.download,
            '-ls': self.list_bucket
        }

    def upload(self, filePath: str, bucket=None):
        bucket = bucket if bucket else self.df_bucket
        filename = filePath.split(dir_char)[-1]
        self.client.upload_file(Bucket=bucket, LocalFilePath=filePath, Key=filename)

    def download(self, filename: str, bucket=None):
        from QuickStart_Rhy.NetTools.normal_dl import normal_dl
        bucket = bucket if bucket else self.df_bucket
        url = 'https://' + bucket + '.cos.' + self.region + '.myqcloud.com/' + filename
        normal_dl(url)

    def remove(self, filePath: str, bucket=None):
        bucket = bucket if bucket else self.df_bucket
        self.client.delete_object(bucket, filePath)

    def list_bucket(self, bucket=None):
        from QuickStart_Rhy.NetTools.normal_dl import size_format
        from prettytable import PrettyTable
        bucket = bucket if bucket else self.df_bucket
        ls = self.client.list_objects(Bucket=bucket)['Contents']
        tb = PrettyTable(['File', 'Size'])
        for obj in ls:
            tb.add_row([obj['Key'], size_format(int(obj['Size']), align=True)])
        print('Bucket Url: https://' + bucket + '.cos.' + self.region + '.myqcloud.com/')
        print(tb)



from QuickStart_Rhy.API import *
import requests


def rmbg(filePath: str):
    api_key = pre_check('rmbg')
    res = requests.post(
        'https://api.remove.bg/v1.0/removebg',
        files={'image_file': open(filePath, 'rb')},
        data={'size': 'auto'},
        headers={'X-Api-Key': api_key},
    )
    if res.status_code == requests.codes.ok:
        img_name = filePath.split(dir_char)[-1].split('.')[0]
        if dir_char in filePath:
            img_root = dir_char.join(filePath.split(dir_char)[:-1]) + dir_char
        else:
            img_root = ''
        with open(img_root + img_name + '_rmbg.png', 'wb') as imgfile:
            imgfile.write(res.content)
    else:
        print('ERROR:', res.status_code, res.text)


def smms(filePath: str):
    from prettytable import PrettyTable
    api_key = pre_check('smms')

    def post_img(path):
        headers = {
            'Authorization': api_key,
        }
        try:
            data = {
                'smfile': (path.split('/')[-1], open(path, 'rb')),
                'format': 'json'
            }
        except:
            return False
        res_json = requests.post('https://sm.ms/api/v2/upload', headers=headers, files=data).text
        return json.loads(res_json)

    def get_path(rt, rel):
        return os.path.abspath(rt + rel)

    def format_markdown(path):
        import re
        _user_path = os.path.expanduser('~')
        rt_path = dir_char.join(os.path.abspath(path).split(dir_char)[:-1]) + dir_char
        res_tb = PrettyTable()
        res_tb.field_names = ['File', 'Status', 'url']
        img_set = {}
        with open(path, 'r') as fp:
            ct = fp.read()
        aims = re.findall('!\[.*?\]\((.*?)\)', ct, re.M)
        for aim in aims:
            raw_path = aim
            aim = aim.replace('~', _user_path)
            aim = aim if aim.startswith(dir_char) else get_path(rt_path, aim)
            if aim not in img_set:
                res_dict = post_img(aim)
                if not res_dict:
                    res_tb.add_row([aim.split(dir_char)[-1], 'No File', ''])
                    img_set[aim] = False
                else:
                    res_tb.add_row(
                        [aim.split(dir_char)[-1], res_dict['success'],
                         res_dict['message'] if not res_dict['success'] else res_dict['data']['url']]
                    )
                    if not res_dict['success'] and res_dict['code'] == 'unauthorized':
                        break
                    img_set[aim] = res_dict['data']['url'] if res_dict['success'] else False
            if img_set[aim]:
                ct = ct.replace(raw_path, img_set[aim])
        with open(path, 'w') as fp:
            fp.write(ct)
        print(res_tb)

    try:
        is_md = filePath.endswith('.md')
    except IndexError:
        exit('Usage: qs {*.md} | {picture}')
    else:
        if is_md:
            format_markdown(filePath)
        else:
            res = post_img(filePath)
            tb = PrettyTable(['File', 'Status', 'url'])
            if not res:
                tb.add_row([filePath.split(dir_char)[-1], 'No File', ''])
            else:
                tb.add_row(
                    [filePath.split(dir_char)[-1], res['success'], '' if not res['success'] else res['data']['url']])
            print(tb)


def pasteme(key: str = '100', password: str = '', mode: str = 'get'):
    import pyperclip
    from colorama import Fore, Style
    if mode == 'get':
        if password:
            r = requests.get('https://api.pasteme.cn/%s,%s' % (key, password), params={'json': True})
        else:
            r = requests.get('https://api.pasteme.cn/%s' % key, params={'json': True})
        if r.status_code == requests.codes.ok:
            js = json.loads(r.content)
            if js['status'] == 200:
                pyperclip.copy(js['content'])
                with open("%s.%s" % (key, js['lang']), 'w') as file:
                    file.write(js['content'])
            else:
                print(Fore.RED, 'Wrong Password' if password else 'Password is required', Style.RESET_ALL)
        else:
            print(Fore.RED, 'Unknown error', Style.RESET_ALL)
    else:
        ss = pyperclip.paste()
        js = {
            'lang': key if key else 'txt',
            'content': ss
        }
        if password:
            js['password'] = password
        r = requests.post('https://api.pasteme.cn',
                          headers={'Content-Type': 'application/json'},
                          json=js)
        if r.status_code == 201:
            print(json.loads(r.content))
        else:
            print('post failed')


def upimg(filePath: str, plt_type: str = 'Ali'):
    from prettytable import PrettyTable

    def post_img(path):
        try:
            data = {'type': plt_type}
            file = [('image', open(path, 'rb'))]
        except:
            return False
        res_json = requests.post('https://v1.alapi.cn/api/image',
                                 data=data, files=file).text
        return json.loads(res_json)

    def get_path(rt, rel):
        return os.path.abspath(rt + rel)

    def format_markdown(path):
        import re
        _user_path = os.path.expanduser('~')
        rt_path = dir_char.join(os.path.abspath(path).split(dir_char)[:-1]) + dir_char
        res_tb = PrettyTable()
        res_tb.field_names = ['File', 'Status', 'url']
        img_set = {}
        with open(path, 'r') as fp:
            ct = fp.read()
        aims = re.findall('!\[.*?\]\((.*?)\)', ct, re.M)
        for aim in aims:
            raw_path = aim
            aim = aim.replace('~', _user_path)
            aim = aim if aim.startswith(dir_char) else get_path(rt_path, aim)
            if aim not in img_set:
                res_dict = post_img(aim)
                if not res_dict:
                    res_tb.add_row([aim.split(dir_char)[-1], 'No File', ''])
                    img_set[aim] = False
                else:
                    res_tb.add_row([aim.split(dir_char)[-1], res_dict['msg'],
                                    '' if res['code'] != 200 else res_dict['data']['url'][plt_type]])
                    if res_dict['code'] != 200:
                        break
                    img_set[aim] = res_dict['data']['url'][plt_type] if res_dict['code'] != 200 else False
            if img_set[aim]:
                ct = ct.replace(raw_path, img_set[aim])
        with open(path, 'w') as fp:
            fp.write(ct)
        print(res_tb)

    try:
        is_md = filePath.endswith('.md')
    except IndexError:
        exit('Usage: qs {*.md} | {picture}')
    else:
        if is_md:
            format_markdown(filePath)
        else:
            res = post_img(filePath)
            tb = PrettyTable(['File', 'Status', 'url'])
            if not res:
                tb.add_row([filePath.split(dir_char)[-1], 'No File', ''])
            else:
                tb.add_row([filePath.split(dir_char)[-1], res['msg'],
                            '' if res['code'] != 200 else res['data']['url'][plt_type]])
            print(tb)



import os
from QuickStart_Rhy import dir_char


def VideoWrapper(func):
    def wrapper(path, *args, **kwargs):
        try:
            import moviepy
        except ImportError as e:
            e = repr(e)
            exit(e)
        else:
            if not os.path.exists(path):
                exit('No such file: %s' % path)
            if not os.path.isfile(path):
                exit('%s not a file!' % path)
            func(path, *args, **kwargs)

    return wrapper


@VideoWrapper
def video_2_gif(path, size=(480, 320), fps=None):
    import moviepy.editor as mpy
    file_name = '.'.join(os.path.basename(path).split('.')[:-1]) + '.gif'
    dir_name = dir_char.join(os.path.abspath(path).split(dir_char)[:-1]) + dir_char
    ct = mpy.VideoFileClip(path) if not size else mpy.VideoFileClip(path).resize(size)
    ct.write_gif(dir_name + file_name, fps=fps) if fps else ct.write_gif(dir_char + file_name)


@VideoWrapper
def rm_audio(path):
    import moviepy.editor as mpy
    file_name = os.path.basename(path)
    if '.' in file_name:
        indx = file_name.index('.')
        file_name = file_name[:indx] + '_rm_audio' + file_name[indx:]
        file_name = file_name.split('.')
        file_name = '.'.join(file_name[:-1]) + '.mp4'
    else:
        file_name += '_rm_audio'
    dir_name = dir_char.join(os.path.abspath(path).split(dir_char)[:-1]) + dir_char
    ct = mpy.VideoFileClip(path).set_audio(None)
    ct.write_videofile(dir_name + file_name)


@VideoWrapper
def tomp4(path):
    import moviepy.editor as mpy
    file_name = os.path.basename(path)
    file_name = '.'.join(file_name[:-1]) + '.mp4' if '.' in file_name else file_name + '.mp4'
    dir_name = dir_char.join(os.path.abspath(path).split(dir_char)[:-1]) + dir_char
    ct = mpy.VideoFileClip(path)
    ct.write_videofile(dir_name + file_name)



from QuickStart_Rhy.API import *
import oss2


class Aliyun_oss_api:
    def __init__(self):
        self.ac_id = pre_check('aliyun_oss_acid')
        self.ac_key = pre_check('aliyun_oss_ackey')
        self.bucket_url = pre_check('aliyun_oss_bucket_url')
        self.df_bucket = pre_check('aliyun_oss_df_bucket')
        self.auth = oss2.Auth(self.ac_id, self.ac_key)

    def get_func_table(self):
        return {
            '-up': self.upload,
            '-rm': self.remove,
            '-dl': self.download,
            '-ls': self.list_bucket
        }

    def upload(self, filePath: str, bucket=None):
        bucket = bucket if bucket else self.df_bucket
        bucket = oss2.Bucket(self.auth, self.bucket_url, bucket)
        oss2.resumable_upload(bucket, filePath.split(dir_char)[-1], filePath, num_threads=4)

    def download(self, filename: str, bucket=None):
        bucket = bucket if bucket else self.df_bucket
        bucket = oss2.Bucket(self.auth, self.bucket_url, bucket)
        oss2.resumable_download(bucket, filename, filename, num_threads=4)

    def remove(self, filePath: str, bucket=None):
        bucket = bucket if bucket else self.df_bucket
        bucket = oss2.Bucket(self.auth, self.bucket_url, bucket)
        bucket.delete_object(filePath)

    def list_bucket(self, bucket=None):
        from QuickStart_Rhy.NetTools.normal_dl import size_format
        from prettytable import PrettyTable
        bucket = bucket if bucket else self.df_bucket
        ls = oss2.Bucket(self.auth, self.bucket_url, bucket)
        tb = PrettyTable(['File', 'Size'])
        for obj in oss2.ObjectIterator(ls):
            tb.add_row([obj.key, size_format(obj.size)])
        print('Bucket Url:', 'https://'+bucket+'.'+self.bucket_url+'/')
        print(tb)



from socketserver import ThreadingMixIn
from http.server import SimpleHTTPRequestHandler
from http.server import HTTPServer


class HttpServers:
    import qrcode_terminal
    import signal

    def __init__(self, ip='localhost', port=8000, url=''):
        self.web_address = ip
        self.web_port = port
        self.httpd = None
        self.bind_url = url

    class Server(ThreadingMixIn, HTTPServer):
        pass

    def start(self):
        HttpServers.signal.signal(HttpServers.signal.SIGINT, self.shutdown)
        self.httpd = HttpServers.Server((self.web_address, self.web_port), SimpleHTTPRequestHandler)
        if not self.bind_url:
            self.bind_url = 'http://' + self.web_address + ':' + str(self.web_port)
        print(self.bind_url)
        HttpServers.qrcode_terminal.draw(self.bind_url)
        try:
            self.httpd.serve_forever()
        except KeyboardInterrupt:
            self.shutdown(0, 0)

    def shutdown(self, a, b):
        self.httpd.server_close()
        print('HTTP Server: Closed.')
        exit(0)



from QuickStart_Rhy.API import pre_check
try:
    import aip
except ImportError:
    exit('You need install: baidu-aip')


class AipNLP:
    def __init__(self, 
                 appid=pre_check("AipNlpAPP_ID"),
                 appkey=pre_check("AipNlpAPP_KEY"),
                 sckey=pre_check("AipNlpSECRET_KEY")):
        self.client = aip.AipNlp(appid, appkey, sckey)

    def get_res(self, words):
        try:
            res = self.client.ecnet(words)['item']['correct_query']
        except Exception as e:
            exit(repr(e))
        else:
            return res



from QuickStart_Rhy import dir_char
import json
import os


user_root = os.path.expanduser('~') + dir_char
if os.path.exists(user_root + '.qsrc'):
    with open(user_root + '.qsrc', 'r', encoding='utf8') as f:
        qsconfig = json.loads(f.read(), encoding='utf8')
else:
    with open(user_root + '.qsrc', 'w') as f:
        f.write('{\n'
                '  "rmbg": "GET API KEY: https://www.remove.bg"\n'
                '  "smms": "GET API KEY: https://sm.ms"\n'
                '}')
    qsconfig = {}


def pre_check(funcName: str, ext=True):
    try:
        api_key = qsconfig[funcName]
        if not api_key:
            exit('You should set %s api key at: %s' % (funcName, user_root + dir_char + '.qsrc'))
    except KeyError:
        if ext:
            exit('You should set %s api key at: %s' % (funcName, user_root + dir_char + '.qsrc'))
        else:
            return False
    else:
        return api_key
